package uoc.ows.server;

import it.sauronsoftware.cron4j.Scheduler;
import uoc.ows.bbdd.*;
import uoc.ows.beans.*;
import uoc.ows.common.CustomRecordFormatter;
import uoc.ows.common.OWSException;
import uoc.ows.common.Constants;

import javax.jms.*;

import java.util.Iterator;
import java.util.logging.Logger;
import java.util.logging.ConsoleHandler;
import java.net.InetAddress;

/**
 * ServerManager
 * s tracta de la classe principal que dona forma al scheduler (planificador i executor de treballs)
 * Treballa a mode de daemon i en cada cicle intercepta el treball a realitzar, tant en termes de planificaci,
 * execuci, desplanificaci, replanificaci i recuperaci de respostes per tal dels nodes del sistema d'informaci
 * 
 * @author LLuis Cozar Prez
 *
 */
public class ServerManager {
	
	
	private static Scheduler scheduler = new Scheduler();
	private static GestorJobs gu;
	private static GestorNodes gn;
	private static GestorTransitions gt;
	private static Logger logger = Logger.getLogger(ServerManager.class.getName());
	private static String IP ;
	
	/**
	 * The main method
	 *
	 * @param args
	 * @throws Exception
	 */
	public static void main(String[] args) throws Exception {

		// Init IP server's address
		IP = InetAddress.getLocalHost().getHostAddress();
		
		// Add ConsoleHandler to Logger.
		CustomRecordFormatter formatter = new CustomRecordFormatter();
		ConsoleHandler consoleHandler = new ConsoleHandler();
		consoleHandler.setFormatter(formatter);
		logger.setUseParentHandlers(false);
		logger.addHandler(consoleHandler);
		logger.info("Inicialitzant server "+IP);
		logger.info("Creaci scheduler OK");

		gu = new GestorJobs();
		gu.conectarBD();
		gn = new GestorNodes(gu.getConexion());
		gt = new GestorTransitions(gu.getConexion());
		
		logger.info("Conexion MySQL OK");

		MySchedulerListener listener = new MySchedulerListener(gu);
		scheduler.addSchedulerListener(listener);
		logger.info("SchedulerListener afegit");
		scheduler.start();
		logger.info("Arranc scheduler OK");
		
		if (args.length > 0) {
			logger.info("Inicialitzacio de la BBDD en mode "+args[0]);
			gu.initStart(args[0]);		
		} else {
			logger.info("Inicialitzacio de la BBDD en mode HOT");
			gu.initStart("HOT");
		}
		
		int index = 1;
		while (index > 0) {

			pendingJobs(); 
			gettingExecutions();
			
			logger.info("Hearbeat cicle scheduler ("+index+")");

			try {
				Thread.sleep(10000L);
			} catch (InterruptedException e) {
				;
			}
			
		}

		scheduler.stop();
		gu.cerrarConexionBD();

	}
	
	/**
	 * pendingJobs
	 * Es tracta del cicle de procs en es planifiquen els treballs pendents, es soliciten les execucions inmediates, 
	 * es desplanifiquen els jobs, etc... tot en funci de l'estat en el que es trobin els jobs en la BBDD
	 * 
	 * @throws OWSException
	 */
	private static void pendingJobs() throws OWSException {

		logger.info("Cicle de cerca de planificacions pendents iniciat");
		Iterator<Job> iterator = gu.getPendingJobs().iterator();
		while (iterator.hasNext()) {
			Job wrkJob = new Job();
			wrkJob = iterator.next();
			
			if (wrkJob.getIdstatus().compareTo(Constants.PENDING) == 0) {

				ActiveMqTask task = new ActiveMqTask(wrkJob);
				wrkJob.setOffsetjob((scheduler.schedule(wrkJob.getPattern(),
						task)));

				MessageSimpleMQ msgMQ = new MessageSimpleMQ();
				msgMQ.setIdjob(wrkJob.getIdjob());
				msgMQ.setSenderNode(IP);
				msgMQ.setQnameReply(Constants.qnameReply);
				msgMQ.setReceiverNode(gn.getNodeDNS(wrkJob.getIdnode()));
				msgMQ.setQname(Constants.qname);
				msgMQ.setCommand(wrkJob.getExecutionCmd());
				msgMQ.setParameters(wrkJob.getExecutionParm());
				msgMQ.setEnvironment(wrkJob.getExecutionEnv());
				msgMQ.setOffsetjob(wrkJob.getOffsetjob());
				

				task.setMessagesMQ(msgMQ);

				wrkJob.setIdstatus(Constants.SCHEDULED);

				gu.updateJob(wrkJob);
				gt.insertTransition(wrkJob);

				logger.info("Job " + wrkJob.getIdjob() + " scheduled ("
						+ wrkJob.toString());

			} else if (wrkJob.getIdstatus().compareTo(Constants.DESCHEDULING) == 0) {
			
				scheduler.deschedule(wrkJob.getOffsetjob());
				wrkJob.setIdstatus(Constants.DESCHEDULED);

				gu.updateJob(wrkJob);
				gt.insertTransition(wrkJob);

				logger.info("Job " + wrkJob.getIdjob() + " descheduled ("
						+ wrkJob.toString());
				
			} else if (wrkJob.getIdstatus().compareTo(Constants.RESCHEDULING) == 0) {
				
				scheduler.reschedule(wrkJob.getOffsetjob(), wrkJob.getPattern());
				wrkJob.setIdstatus(Constants.RESCHEDULED);

				gu.updateJob(wrkJob);
				gt.insertTransition(wrkJob);

				logger.info("Job " + wrkJob.getIdjob() + " rescheduled ("
						+ wrkJob.toString());
				
			} else if (wrkJob.getIdstatus().compareTo(Constants.EXECUTE) == 0) {
				
				scheduler.launch(scheduler.getTask(wrkJob.getOffsetjob()));
				logger.info("Job " + wrkJob.getIdjob() + " executat inmediat ("
					+ wrkJob.toString());
			
			}
		}

		logger.info("Cicle de cerca de planificacions finalitzat");
	}
	
	/**
	 * gettingExecutions
	 * Es tracta de la funci que recuperar el missatges de finalitzaci dels jobs i els stdout/err de la resta de nodes 
	 * del sistema d'informaci
 	 * 
	 * @throws JMSException
	 * @throws OWSException
	 */
	private static void gettingExecutions() throws JMSException, OWSException {
		
		final MessageReceiver userActionConsumer = new MessageReceiver(gu);
		
		logger.info("Cicle de lectura d'execucions iniciat");
        userActionConsumer.processMessages();
        logger.info("Cicle de lectura d'execucions finalitzat");
	}
}
